%
%   [lamb_all,lamb,lamb_err,unit,eps,b,Mmax,err]=TruncGR(t,M,iop,Mmin)
%
% ESTIMATES THE MEAN ACTIVITY RATE WITHIN THE WHOLE SAMPLE AND WITHIN THE 
% PART OF THE SAMPLE COMPRISING EVENTS >=Mmin (COMPLETE PART), 
%THE ROUND-OFF ERROR OF MAGNITUDE, THE GUTENBERG-RICHTER B-VALUE 
% AND THE UPPER BOUND OF MAGNITUDE DISTRIBUTION USING THE UPPER-BOUNDED 
% G-R LED MAGNITUDE DISTRIBUTION MODEL
%
% AUTHOR: Stanislaw. Lasocki, Institute of Geophysics Polish Academy of
% Sciences, Warsaw, Poland
%
% DESCRIPTION: The assumption on the upper-bounded Gutenberg-Richter 
% relation leads to the upper truncated exponential distribution to model 
% magnitude distribution from and above the catalog completness level 
% Mmin. The shape parameter of this distribution and consequently the G-R
% b-value is estimated by maximum likelihood method (Aki-Utsu procedure).   
% The upper limit of the distribution Mmax is evaluated using 
% the Kijko-Sellevol generic formula. If convergence is not reached the
% Whitlock @ Robson simplified formula is used: 
% Mmaxest= 2(max obs M) - (second max obs M)).
% The mean activity rate, lamb, is the number of events >=Mmin into the
% length of the period in which they occurred. Upon the value of the input 
% parameter, iop, the used unit of time can be either day ot month or year.    
% The round-off interval length - eps is the least non-zero difference 
% between sample data or 0.1 if the least difference is greater than 0.1.
%
% REFERENCES:
%Kijko, A., and M.A. Sellevoll (1989) Bull. Seismol. Soc. Am. 79, 3,645-654 
%Lasocki, S., Urban, P. (2011) Acta Geophys 59, 659-673, 
% doi: 10.2478/s11600-010-0049-y
%
% INPUT:
%   t - vector of earthquake occurrence times
%   M - vector of magnitudes from a user selected catalog
%   iop - determines the used unit of time. iop=0 - 'day', iop=1 - 'month', 
%       iop=2 - 'year'
%   Mmin - catalog completeness level. Can take any value from [min(M), max(M)].
%
% OUTPUT:
%
%   lamb_all - mean activity rate for all events
%   lamb - mean activity rate for events >= Mmin
%   lamb_err - error paramter on the number of events >=Mmin. lamb_err=0
%       for 15 or more events >=Mmin and the parameter estimation is
%       continued, lamb_err=1 otherwise, all output paramters except 
%       lamb_all and lamb are set to zero and the function execution is 
%       terminated.  
%   unit - string with name of time unit used ('year' or 'month' or 'day').
%   eps - length of the round-off interval of magnitudes.
%   b - Gutenberg-Richter b-value
%   Mmax - upper limit of magnitude distribution
%   err - error parameter on Mmax estimation, err=0 - convergence, err=1 - 
%   no converegence of Kijko-Sellevol estimator, Robinson @ Whitlock
%   method used.
%
% LICENSE
%     This file is a part of the IS-EPOS e-PLATFORM.
%
%     This is free software: you can redistribute it and/or modify it under 
%     the terms of the GNU General Public License as published by the Free 
%     Software Foundation, either version 3 of the License, or 
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details, <http://www.gnu.org/licenses/>.
% 

function [lamb_all,lamb,lamb_err,unit,eps,b,Mmax,err]=TruncGR(t,M,iop,Mmin)
n=length(M);
lamb_err=0;
t1=t(1);
for i=1:n
    if M(i)>=Mmin; break; end
    t1=t(i+1);
end
    t2=t(n);
for i=n:1
    if M(i)>=Mmin; break; end
    t2=t(i-1);
end
nn=0;
for i=1:n
    if M(i)>=Mmin
        nn=nn+1;
    end
end
    
if iop==0
    lamb_all=n/round(t(n)-t(1));
    lamb=nn/round(t2-t1);
    unit='day';
elseif iop==1
lamb_all=30*n/(t(n)-t(1));      
lamb=30*nn/(t2-t1);             
    unit='month';
else
lamb_all=365*n/(t(n)-t(1));    
lamb=365*nn/(t2-t1);               
    unit='year';
end

if nn<15
    eps=0;b=0;Mmax=0;err=0;
    lamb_err=1;
    return;
end

eps=magn_accur(M);
xx=M(M>=Mmin);
clear x;
nn=length(xx);
Max_obs=max(xx);
beta0=0;
Mmax1=Max_obs;
for i=1:50,
    beta=fzero(@bet_est,[0.05,4.0],[],mean(xx),Mmin-eps/2,Mmax1);
  	Mmax=Max_obs+moja_calka('f_podc',Mmin,Max_obs,1e-5,nn,beta,Mmin-eps/2,Mmax1);
    if ((abs(Mmax-Mmax1)<0.01)&&(abs(beta-beta0)<0.0001))
        err=0;
        break;
    end
    Mmax1=Mmax;
    beta0=beta;
end
if i==50; 
    err=1.0;
    Mmax=2*xx(1)-xx(2);
    beta=fzero(@bet_est,1.0,[],mean(xx),Mmin-eps/2,Mmax);
end
b=beta/log(10);
clear xx
end

function [zero]=bet_est(b,ms,Mmin,Mmax)

%First derivative of the log likelihood function of the upper-bounded 
% exponential distribution (truncated GR model)
% b - parameter of the distribution 'beta'
% ms - mean of the observed magnitudes
% Mmin - catalog completeness level
% Mmax - upper limit of the distribution

M_max_min=Mmax-Mmin;
e_m=exp(-b*M_max_min);
zero=1/b-ms+Mmin-M_max_min*e_m/(1-e_m);

end


function [calka,ier]=moja_calka(funfc,a,b,eps,varargin)

% Integration by means of 16th poit Gauss method. Adopted from CERNLIBRARY

% funfc - string with the name of function to be integrated
% a,b - integration limits
% eps - accurracy
% varargin - other parameters of function to be integrated
% calka - integral
% ier=0 - convergence, ier=1 - no conbergence

persistent W X CONST
W=[0.101228536290376 0.222381034453374 0.313706645877887 ...
0.362683783378362 0.027152459411754 0.062253523938648 ...
0.095158511682493 0.124628971255534 0.149595988816577 ...
0.169156519395003 0.182603415044924 0.189450610455069];
X=[0.960289856497536 0.796666477413627 0.525532409916329 ...
0.183434642495650 0.989400934991650 0.944575023073233 ...
0.865631202387832 0.755404408355003 0.617876244402644 ...
0.458016777657227 0.281603550779259 0.095012509837637];
CONST=1E-12;
delta=CONST*abs(a-b);
calka=0.;
aa=a;
y=b-aa;
ier=0;
while abs(y)>delta,
   bb=aa+y;
   c1=0.5*(aa+bb);
   c2=c1-aa;
   s8=0.;
   s16=0.;
   for i=1:4,
      u=X(i)*c2;
      s8=s8+W(i)*(feval(funfc,c1+u,varargin{:})+feval(funfc,c1-u,varargin{:}));
   end
   for i=5:12,
      u=X(i)*c2;
      s16=s16+W(i)*(feval(funfc,c1+u,varargin{:})+feval(funfc,c1-u,varargin{:}));
   end
   s8=s8*c2;
   s16=s16*c2;
   if abs(s16-s8)>eps*(1+abs(s16))
      y=0.5*y;
      calka=0.;
      ier=1;
   else
      calka=calka+s16;
      aa=bb;
      y=b-aa;
      ier=0;
   end
end
end

function [y]=f_podc(z,n,beta,Mmin,Mmax)

% Integrated function for Mmax estimation. Truncated GR model
% z - column vector of independent variable
% n - the size of 'z'
% beta - the distribution parameter
% Mmin - the catalog completeness level
% Mmax - the upper limit of the distribution

y=Cdfgr(z,beta,Mmin,Mmax).^n;
end

function [y]=Cdfgr(t,beta,Mmin,Mmax)

%CDF of the truncated upper-bounded exponential distribution (truncated G-R
% model
% Mmin - catalog completeness level
% Mmax - upper limit of the distribution
% beta - the distribution parameter
% t - vector of magnitudes (independent variable)
% y - CDF vector

mian=(1-exp(-beta*(Mmax-Mmin)));
y=(1-exp(-beta*(t-Mmin)))/mian;
idx=find(y>1);
y(idx)=ones(size(idx));
end

function [eps]=magn_accur(M)
x=sort(M);
d=x(2:length(x))-x(1:length(x)-1);
eps=min(d(d>0));
if eps>0.1; eps=0.1;end
end
